#include "filtering.h"

#include "kadu.h"
#include "userbox.h"
#include "usergroup.h"
#include "userlist.h"
#include "debug.h"
#include "misc.h"
#include "config_dialog.h"
#include "tabbar.h"

#include <qlayout.h>
#include <qobjectlist.h>

extern "C" int filtering_init()
{
	kdebugf();
	
	filtering = new Filtering ();
	filtering->setShown (config_file.readBoolEntry ("filtering", "filter-showonload", false));
	
	kdebugf2();
	return 0;
}

extern "C" void filtering_close()
{
	kdebugf();
	
	delete filtering;
	filtering = NULL;
	
	kdebugf2();
}

const char * LABELTEXT = "Filtering, by default, uses contact's first name, last name,\nnick name and alternative display nick name.\nThis is done by using 'contains' method.";

Filtering::Filtering(): QWidget(kadu, "filtering")
{
	kdebugf();
	
	kadu->mainLayout ()->insertWidget (2, this);
	
	ConfigDialog::addHotKeyEdit ("ShortCuts", "Define keys", QT_TRANSLATE_NOOP("@default", "Filter contacts"), "filtering_start", "/");
	ConfigDialog::addTab ("filtering", dataPath ("kadu/modules/data/filtering/filtering.png"));
	ConfigDialog::addCheckBox (
		"filtering",
		"filtering",
		QT_TRANSLATE_NOOP("@default", "Show after loading module"),
		"filter-showonload",
		true
	);
	ConfigDialog::addCheckBox (
		"filtering",
		"filtering",
		QT_TRANSLATE_NOOP("@default", "Any keypress will start contacts list filtering"),
		"filter-startonany",
		false
	);
	ConfigDialog::addLabel("filtering", "filtering", QT_TRANSLATE_NOOP("@default", "Filtering, by default, uses contact's first name, last name,\nnick name and alternative display nick name.\nThis is done by using 'contains' method."));
	ConfigDialog::addCheckBox (
		"filtering",
		"filtering",
		QT_TRANSLATE_NOOP("@default", "Use 'starts with' method"),
		"filter-startswith",
		false,
		QT_TRANSLATE_NOOP("@default", "Entered text will be searched for from the beginning")
	);
	ConfigDialog::addCheckBox (
		"filtering",
		"filtering",
		QT_TRANSLATE_NOOP("@default", "Search also in contact's number"),
		"filter-number",
		false,
		QT_TRANSLATE_NOOP("@default", "Contact's number is GG user number")
	);
	ConfigDialog::addCheckBox (
		"filtering",
		"filtering",
		QT_TRANSLATE_NOOP("@default", "Search also in contact's email"),
		"filter-email",
		false/*,
		QT_TRANSLATE_NOOP("@default", "")*/
	);
	ConfigDialog::addCheckBox (
		"filtering",
		"filtering",
		QT_TRANSLATE_NOOP("@default", "Search also in contact's mobile phone number"),
		"filter-mobile",
		false/*,
		QT_TRANSLATE_NOOP("@default", "")*/
	);
	
	filter = new UserGroup(userlist->toUserListElements(), "filter");
	
	QHBoxLayout *l = new QHBoxLayout(this);

	l->addWidget (clearPB = new QPushButton(this, "clearPB"));
	clearPB->setPixmap (QPixmap (dataPath ("kadu/modules/data/filtering/clear.png")));
	l->addWidget (new QLabel (tr("Filter"), this, "filterLBL"));
	l->addWidget (textLE = new QLineEdit(this, "textLE"));
	
	connect(clearPB, SIGNAL(clicked()), this, SLOT(on_clearPB_clicked()));
	connect(textLE, SIGNAL(textChanged(const QString&)), this, SLOT(on_textLE_textChanged(const QString&)));
	connect(textLE, SIGNAL(returnPressed()), this, SLOT(on_textLE_returnPressed()));
	connect(kadu, SIGNAL(keyPressed(QKeyEvent*)), this, SLOT(on_kadu_keyPressed(QKeyEvent*)));

	kadu->userbox ()->installEventFilter (this);
	
	kdebugf2();
}
	
Filtering::~Filtering()
{
	kdebugf();
	
	if (! textLE->text ().isEmpty ()) {
		kadu->userbox ()->removeFilter (filter);
	}
	
	disconnect(clearPB, SIGNAL(clicked()), this, SLOT(on_clearPB_clicked()));
	disconnect(textLE, SIGNAL(textChanged(const QString&)), this, SLOT(on_textLE_textChanged(const QString&)));
	disconnect(kadu, SIGNAL(keyPressed(QKeyEvent*)), this, SLOT(on_kadu_keyPressed(QKeyEvent*)));

	kadu->userbox ()->removeEventFilter (this);
	
	ConfigDialog::removeControl ("ShortCuts", "Filter contacts");
	ConfigDialog::removeControl ("filtering", LABELTEXT);
	ConfigDialog::removeControl ("filtering", "Show after loading module");
	ConfigDialog::removeControl ("filtering", "Any keypress will start contacts list filtering");
	ConfigDialog::removeControl ("filtering", "Use 'starts with' method");
	ConfigDialog::removeControl ("filtering", "Search also in contact's number");
	ConfigDialog::removeControl ("filtering", "Search also in contact's email");
	ConfigDialog::removeControl ("filtering", "Search also in contact's mobile phone number");
	ConfigDialog::removeTab ("filtering");
	
	kdebugf2();
}

void Filtering::on_kadu_keyPressed (QKeyEvent *e)
{
	kdebugf();
	
	bool startonany = config_file.readBoolEntry ("filtering", "filter-startonany", false);
	bool shortcut = HotKey::shortCut (e, "ShortCuts", "filtering_start");
	
	if ((shortcut || startonany) && kadu->userbox ()->count ()) {
		show ();
		textLE->setFocus ();
		if (startonany && ! shortcut) {
			textLE->setText (e->text ());
		}
	}
	
	kdebugf2();
}

bool Filtering::eventFilter (QObject *o, QEvent *e)
{
	kdebugf();
	
	if (e->type () == QEvent::KeyPress) {
		QKeyEvent *k = (QKeyEvent*) e;
		on_kadu_keyPressed (k);
	}
	
	kdebugf2();
	return false;
}

void Filtering::keyPressEvent (QKeyEvent *e)
{
	kdebugf();
	
	if (e->key () == Qt::Key_Escape) {
		hide ();
		clearFilter ();
		e->accept ();
		kadu->userbox ()->setFocus ();
	} else {
		e->ignore ();
	}
	
	kdebugf2();
}

void Filtering::on_clearPB_clicked ()
{
	kdebugf();
	
	textLE->clear ();
	
	kdebugf2();
}

void Filtering::on_textLE_textChanged (const QString& s)
{
	kdebugf();
	
	clearFilter ();
	
	if (! s.isEmpty ())
		filterWith (s);
	
	kdebugf2();
}

void Filtering::clearFilter ()
{
	kdebugf();
	
	kadu->userbox ()->removeFilter (filter);
	filter->removeUsers (userlist);
	
	kdebugf2();
}

void Filtering::filterWith (const QString& f)
{
	kdebugf();
	
	bool filter_number = config_file.readBoolEntry ("filtering", "filter-number", false);
	bool filter_email = config_file.readBoolEntry ("filtering", "filter-email", false);
	bool filter_mobile = config_file.readBoolEntry ("filtering", "filter-mobile", false);
	bool filter_startswith = config_file.readBoolEntry ("filtering", "filter-startswith", false);
	
	CONST_FOREACH(u, *userlist) {
		if (checkString ((*u).firstName (), f, filter_startswith)
				|| checkString ((*u).lastName (), f, filter_startswith)
				|| checkString ((*u).altNick (), f, filter_startswith)
				|| checkString ((*u).nickName (), f, filter_startswith)
				|| (filter_number && (*u).usesProtocol ("Gadu") && checkString ((*u).ID ("Gadu"), f, filter_startswith))
				|| (filter_email && checkString ((*u).email (), f, filter_startswith))
				|| (filter_mobile && checkString ((*u).mobile (), f, filter_startswith))) {
			filter->addUser (*u);
		}
	}
	
	kadu->userbox ()->applyFilter (filter);
	
	kdebugf2();
}

bool Filtering::checkString (const QString &hay, const QString& needle, bool startsWith)
{
	int index = hay.find (needle, 0, false);
	
	return /* znaleziono gdziekolwiek */ (index >= 0) 
			&& (/* startsWith implikuje znaleziono na pocztku*/ startsWith <= (index == 0));
}

void Filtering::on_textLE_returnPressed ()
{
	kadu->userbox ()->setSelected (0, true);
	kadu->userbox ()->setFocus ();
}

Filtering *filtering = 0;
